package fabbroniko.testrawvideofile;

import android.media.MediaCodec;
import android.media.MediaFormat;
import android.support.annotation.NonNull;
import android.util.Log;
import android.view.Surface;

import java.io.IOException;
import java.nio.ByteBuffer;

import fabbroniko.testrawvideofile.communication.Communication;
import fabbroniko.testrawvideofile.communication.TCPCommunication;
import fabbroniko.testrawvideofile.communication.UDPCommunication;
import fabbroniko.testrawvideofile.h264.InputFileVideoCodecH264;

public class DecoderThread{

    private static final byte[] KNOWN_SPS = {0, 0, 0, 1, 39, 100, 0, 40, -44, 43, 64, 98, 27, -119, 60, 3, -68, -9, -40};
    private static final byte[] KNOWN_PPS = {0, 0, 0, 1, 40, -110, 1, 55, 44};
    private MediaCodec mediaCodec = null;
    private Communication comm = new TCPCommunication();
    private int decodecNalUnitsCounter = 0;

    public void init(final Surface surface /*, final InputStream inputStream */) {
        comm.connect();

        try {
            mediaCodec = MediaCodec.createDecoderByType(MediaFormat.MIMETYPE_VIDEO_AVC);
            MainActivity.showMessage("Creato MediaCodec con successo.");
        } catch (final IOException e1) {
            MainActivity.showMessage("Impossibile creare il MediaCodec.");
            e1.printStackTrace();
            System.exit(1);
        } catch (final IllegalArgumentException e2) {
            MainActivity.showMessage("Tipo del MediaCodec non valido.");
            e2.printStackTrace();
            System.exit(1);
        } catch (final NullPointerException e3) {
            MainActivity.showMessage("Passato NULL nella creazione del MediaCodec.");
            e3.printStackTrace();
            System.exit(1);
        }

        mediaCodec.setCallback(new MediaCodec.Callback() {
            @Override
            public void onInputBufferAvailable(@NonNull MediaCodec codec, int index) {
                //System.out.println("InputBuffer disponibile.");
                ByteBuffer inputBuffer = mediaCodec.getInputBuffer(index);
                byte[] nalUnit = comm.getNextNalUnit();

                if(nalUnit != null) {
                    Log.d("Debug", "NAL Unit N. " + ++decodecNalUnitsCounter + " in elaborazione. Timestamp: " + System.currentTimeMillis());
                    //if(InputFileVideoCodecH264.isSPSNALUnit(nalUnit) || InputFileVideoCodecH264.isPPSNALUnit(nalUnit)) {
                        //System.out.println("config");
                        //mediaCodec.queueInputBuffer(index, 0, nalUnit.length, System.nanoTime(), MediaCodec.BUFFER_FLAG_CODEC_CONFIG);
                    //} else {
                        inputBuffer.put(nalUnit);
                        mediaCodec.queueInputBuffer(index, 0, nalUnit.length, System.nanoTime()/*System.currentTimeMillis() * 1000*/, 0);
                    //}
                } else {
                    if(comm.isClosed()) {
                        mediaCodec.queueInputBuffer(index, 0, 0, 0, MediaCodec.BUFFER_FLAG_END_OF_STREAM);
                        close();
                    }
                }
            }

            private int tmpCounterOutput = 0;
            @Override
            public void onOutputBufferAvailable(@NonNull MediaCodec codec, int index, @NonNull MediaCodec.BufferInfo info) {
                Log.d("Debug", "NAL Unit N. " + ++tmpCounterOutput + " finita elaborazione. Timestamp: " + System.currentTimeMillis());
                //System.out.println("outputBufferAvailable");
                if (index == MediaCodec.INFO_TRY_AGAIN_LATER) {
                    MainActivity.showMessage("NO OUTPUT AVAILABLE YET.");
                } else if (index == MediaCodec.INFO_OUTPUT_FORMAT_CHANGED) {
                    MainActivity.showMessage("MediaFormat Changed.");
                } else if (index < 0) {
                    MainActivity.showMessage("Unexpected result from dequeueOutputBuffer.");
                } else {
                    mediaCodec.releaseOutputBuffer(index, true);
                }
            }

            @Override
            public void onError(@NonNull MediaCodec codec, @NonNull MediaCodec.CodecException e) {
                MainActivity.showMessage("MediaCodecException: ");
                e.printStackTrace();
            }

            @Override
            public void onOutputFormatChanged(@NonNull MediaCodec codec, @NonNull MediaFormat format) {}
        });

        final MediaFormat mediaFormat = MediaFormat.createVideoFormat(MediaFormat.MIMETYPE_VIDEO_AVC, 854, 480);
        mediaFormat.setInteger(MediaFormat.KEY_BIT_RATE, 1200000);
        mediaFormat.setInteger(MediaFormat.KEY_FRAME_RATE, 25);
        mediaFormat.setByteBuffer("csd-0", ByteBuffer.wrap(KNOWN_SPS));
        mediaFormat.setByteBuffer("csd-1", ByteBuffer.wrap(KNOWN_PPS));
        mediaCodec.configure(mediaFormat, surface, null, 0);
        mediaCodec.start();
    }

    public void close() {
        comm.close();
    }
}
