package fabbroniko.testrawvideofile.communication;

import android.os.StrictMode;

import java.io.DataInputStream;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.ByteBuffer;

import fabbroniko.testrawvideofile.h264.InputFileVideoCodecH264;

public class TCPCommunication implements Communication {

    private ServerSocket serverSocket;
    private Socket sourceSocket;
    private DataInputStream dataInputStream;

    private boolean communicationActive = false;

    private static final int PORT_NUM = 60000;
    private static final int HEADER_LENGTH = 2;
    private static final ByteBuffer convBuffer = ByteBuffer.allocate((Integer.SIZE / 8));

    byte[] headerBuffer = new byte[HEADER_LENGTH];
    byte[] buffer;

    @Override
    public void connect() {
        StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder().permitAll().build();
        StrictMode.setThreadPolicy(policy);
        try {
            serverSocket = new ServerSocket(PORT_NUM);
            sourceSocket = serverSocket.accept();
            dataInputStream = new DataInputStream(sourceSocket.getInputStream());
            communicationActive = true;
        } catch (final IOException e) {
            e.printStackTrace();
            System.exit(1);
        }
    }

    @Override
    public byte[] getNextNalUnit() {
        try {
            dataInputStream.readFully(headerBuffer);
            buffer = new byte[convertToInt(headerBuffer)];
            dataInputStream.readFully(buffer);

            /*
            if(InputFileVideoCodecH264.isSPSNALUnit(buffer)) {
                byte[] tmp = getNextNalUnit();
                if(InputFileVideoCodecH264.isPPSNALUnit(tmp)) {
                    byte[] multi = new byte[buffer.length + tmp.length];
                    System.arraycopy(buffer, 0, multi, 0, buffer.length);
                    System.arraycopy(tmp, 0, multi, buffer.length, tmp.length);
                    return multi;
                }
            }*/
        } catch (final IOException e) {
            close();
            return null;
        }

        return buffer;
    }

    @Override
    public void close() {
        try {
            dataInputStream.close();
            sourceSocket.close();
            serverSocket.close();
            communicationActive = false;
        }catch (final IOException e) {
            e.printStackTrace();
            System.exit(1);
        }

    }

    @Override
    public boolean isClosed() {
        return communicationActive;
    }

    private static int convertToInt(final byte[] dataLength) {
        if(dataLength.length != 2) {
            throw new IllegalArgumentException("Header length diversa dal previsto.");
        }

        for(int i = 0; i < (Integer.SIZE / 8) - 2; i++) {
            convBuffer.put(i, (byte)0x00);
        }

        convBuffer.put((Integer.SIZE / 8) - 2, dataLength[0]);
        convBuffer.put((Integer.SIZE / 8) - 1, dataLength[1]);

        return convBuffer.getInt(0);
    }
}
