package fabbroniko.testrawvideofile.communication;

import android.util.Log;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.util.LinkedList;

public class UDPCommunication implements Communication, Runnable {

    private boolean connected;
    private DatagramSocket socket;
    private boolean exit;
    private final Sync sync;
    private LinkedListSafe<byte[]> d = new LinkedListSafe<>();
    private int receivedNalUnitsCounter = 0;

    private static final int PORT_NUMBER = 60000;
    private static final int RECEIVE_TIMEOUT = 1000; // milliseconds
    private static final int MAX_PACKET_LENGTH = 65536;

    public UDPCommunication() {
        connected = false;
        exit = false;
        sync = new Sync();
    }

    @Override
    public void connect() {
        if(connected)
            throw new IllegalStateException("Comunicazione già stabilita.");

        try {
            socket = new DatagramSocket(PORT_NUMBER);
            socket.setSoTimeout(RECEIVE_TIMEOUT);
        } catch (final SocketException e) {
            e.printStackTrace();
            System.exit(1);
        }

        connected = true;
        exit = false;

        new Thread(this).start();
    }

    @Override
    public byte[] getNextNalUnit() {
        return d.poll();
    }

    @Override
    public void close() {
        synchronized (sync) {
            exit = true;
        }
    }

    @Override
    public void run() {
        boolean localExit;
        boolean timeout;
        byte[] buffer;

        do {
            timeout = false;
            buffer = new byte[MAX_PACKET_LENGTH];
            final DatagramPacket dp = new DatagramPacket(buffer, MAX_PACKET_LENGTH);

            try {
                socket.receive(dp);
                Log.d("Debug", "Datagram N. " + ++receivedNalUnitsCounter + " Received. TimeStamp " + System.currentTimeMillis());
            }catch (final SocketTimeoutException e1) {
                timeout = true;
            }catch (final IOException e2) {
                e2.printStackTrace();
                System.exit(1);
            }

            if(!timeout)
                d.add(buffer);

            synchronized (sync) {
                localExit = exit;
            }
        } while (!localExit);

        synchronized (sync) {
            connected = false;
        }
    }

    @Override
    public boolean isClosed() {
        boolean localExit;

        synchronized (sync) {
            localExit = exit;
        }

        return localExit;
    }

    private class LinkedListSafe<T> {

        private LinkedList<T> queue;
        private final Sync mySynchro;

        public LinkedListSafe() {
            queue = new LinkedList<>();
            mySynchro = new Sync();
        }

        public T poll() {
            T retValue;

            synchronized (mySynchro) {
                while(queue.size() == 0) {
                    try {
                        mySynchro.wait();
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
                retValue = queue.poll();
            }

            return retValue;
        }

        public int size() {
            int size;

            synchronized (mySynchro) {
                size = queue.size();
            }

            return size;
        }

        public void add(final T element) {
            synchronized (mySynchro) {
                queue.add(element);
                Log.d("Debug", "Numero elementi queue: " + queue.size());
                mySynchro.notify();
            }
        }
    }

    private class Sync {}
}
