package fabbroniko.testrawvideofile.h264;

import java.io.IOException;
import java.io.InputStream;

public abstract class AbstractCodecH264 implements CodecH264 {

    protected final InputStream inputStream;
    protected NALUnit mainBuffer;
    protected NALUnit spsUnit;
    protected NALUnit ppsUnit;
    protected boolean isVideoStreamClosed;

    public AbstractCodecH264(final InputStream inputStream) {
        this.inputStream = inputStream;
        mainBuffer = new NALUnit();
        spsUnit = null;
        ppsUnit = null;
        isVideoStreamClosed = false;
    }

    @Override
    public abstract byte[] getNextNALUnit();

    @Override
    public synchronized boolean isSPSAvailable() {
        return (spsUnit != null) && spsUnit.isSPS();
    }

    @Override
    public synchronized byte[] getSPSNALUnit() {
        return spsUnit.array();
    }

    @Override
    public synchronized boolean isPPSAvailable() {
        return (ppsUnit != null) && spsUnit.isPPS();
    }

    @Override
    public synchronized byte[] getPPSNALUnit() {
        return ppsUnit.array();
    }

    @Override
    public synchronized boolean isStreamClosed() {
        return isVideoStreamClosed;
    }

    @Override
    public synchronized void closeStream() {
        try {
            inputStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
