package fabbroniko.testrawvideofile.h264;

public class NALUnit implements DynamicByteArray {

	public static final int DEFAULT_BUFFER_LENGTH = 50;
    public static final byte[] STARTING_SEQUENCE = {0x00, 0x00, 0x00, 0x01};
    public static final byte H264_SPS_VALUE = 39;
    public static final byte H264_PPS_VALUE = 40;
    public static final int H264_TYPE_INDEX = 4;

	private byte[] buffer;
	private int bufferLength;
	private int writeIndex;

	public NALUnit() {
		this(DEFAULT_BUFFER_LENGTH);
	}

	public NALUnit(final int bufferLength) {
		buffer = new byte[bufferLength];
		this.bufferLength = bufferLength;
		writeIndex = 0;
	}

	@Override
	public void put(final byte b) {
        if(writeIndex >= bufferLength)
            expandBuffer();

		buffer[writeIndex] = b;
		writeIndex++;
	}

	@Override
	public void put(final byte[] b) {
		if(writeIndex + b.length >= bufferLength) {
            expandBuffer();
            put(b);
        } else {
            System.arraycopy(b, 0, buffer, writeIndex, b.length);
        }
	}

	@Override
	public void put(final byte[] b, final int offset, final int length) throws  IllegalArgumentException {
        if((offset >= b.length) || ((offset + length) > b.length))
            throw new IllegalArgumentException("Fuori dai limiti dell'array.");

        if((writeIndex + length) >= bufferLength) {
            expandBuffer();
            put(b, offset, length);
        } else {
            System.arraycopy(b, offset, buffer, writeIndex, length);
        }
	}

	@Override
	public void clear() {
		buffer = new byte[bufferLength];
		writeIndex = 0;
	}

	@Override
	public byte[] array() {
		final byte[] retValue = new byte[writeIndex];
        System.arraycopy(buffer, 0, retValue, 0, writeIndex);
		return retValue;
	}

    private void expandBuffer() {
        final int newLength = bufferLength * 2;
        final byte[] newBuffer = new byte[newLength];

        System.arraycopy(buffer, 0, newBuffer, 0, bufferLength);

        buffer = newBuffer;
        bufferLength = newLength;
    }

    public boolean isSPS() {
        return buffer != null && writeIndex >= STARTING_SEQUENCE.length + 1 && buffer[H264_TYPE_INDEX] == H264_SPS_VALUE;
    }

    public boolean isPPS() {
        return buffer != null && writeIndex >= STARTING_SEQUENCE.length + 1 && buffer[H264_TYPE_INDEX] == H264_PPS_VALUE;
    }
}
